package ft;

import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.sidiff.difference.matcher.BaseMatcher;

import FaultTree.FaultTreePackage;
import FaultTree.Root;

/**
 * Basically NamedElement Matcher, special handling of Root Element and Elements
 * with "id" attribute.
 * 
 * 
 * @author kehrer
 */
public class FtMatcher extends BaseMatcher {

	public static final String KEY = "FtMatcher";

	/**
	 * Initialize matcher and start matching.
	 */
	public FtMatcher() {
		super();
	}

	@Override
	protected boolean isCorresponding(EObject elementA, EObject elementB) {
		assert (elementA != null && elementB != null) : "One of the elements to check for correspondence is null!";

		// type should be identical in order to correspond
		if (elementA.eClass() != elementB.eClass()) {
			return false;
		}

		// None of the elements must be already in a correspondence
		if (isCorresponding(elementA) || isCorresponding(elementB)) {
			return false;
		}

		// "Singleton" elements
		if (elementA instanceof Root) {
			return true;
		}

		assert(hasID(elementA) || hasName(elementA));
		
		// Elements with ID
		// Check for attribute "id" and its values equality
		if (hasID(elementA) && equalIDs(elementA, elementB)){
			return true;
		}
		
		// Named Elements
		// Check for attribute "name" and its values equality
		if (hasName(elementA) && equalNames(elementA, elementB)){
			return true;
		}
		
		return false;
	}

	private boolean hasName(EObject object) {
		EStructuralFeature attrName = object.eClass().getEStructuralFeature("name");
		return attrName != null && attrName instanceof EAttribute;
	}

	private boolean equalNames(EObject elementA, EObject elementB) {
		EStructuralFeature attrName = elementA.eClass().getEStructuralFeature("name");
		Object nameA = elementA.eGet(attrName);
		Object nameB = elementB.eGet(attrName);

		if (nameA != null && nameB != null) {
			return nameA.equals(nameB);
		}

		return false;
	}

	private boolean hasID(EObject object) {
		EStructuralFeature attrId = object.eClass().getEStructuralFeature("id");
		return attrId != null && attrId instanceof EAttribute;
	}

	private boolean equalIDs(EObject elementA, EObject elementB) {
		EStructuralFeature attrId = elementA.eClass().getEStructuralFeature("id");
		Object idA = elementA.eGet(attrId);
		Object idB = elementB.eGet(attrId);

		if (idA != null && idB != null && !idA.equals(attrId.getDefaultValue())) {
			return idA.equals(idB);
		}

		return false;
	}

	@Override
	public String getName() {
		return "FT Matcher";
	}

	@Override
	public String getKey() {
		return KEY;
	}

	@Override
	public String getDocumentType() {
		return FaultTreePackage.eNS_URI;
	}

	@Override
	public boolean canComputeReliability() {
		return false;
	}
}
